#include <LiquidCrystal.h>  //бибилиотека экрана  
#include <OneWire.h>          // для обмена по 1-wire


LiquidCrystal lcd(78, 79, 80, 81, 82, 83); //распиновка дисплея

/* распиновка */
int   mpxPin = 9; //вход MPX
OneWire  ds(2);             // на этот пин подключена линия DQ 1-wire

// переменные
int   mpx;                   // датчик давления
int   mpx_a[2] = {60,75}; // массив для мин и макс значений
byte i;                     // просто счетчик
byte n;                     // просто счетчик
byte m=0;                   // просто счетчик
byte t=0;                   // просто счетчик
float T[]={0,0,0,0};        // переменная для хранения температуры
float Tp[]={0,0,0,0};       // переменная для хранения температуры с прошлого замера нужно для устранения некоторых ошибок
byte data[9];               // массив для хранения данных принятых от датчика
byte TotalDS=0;             // число найденых датчиков на линии
byte addr[4][8] = {         // двумерный массив адресов датчиков
{0x28,0x36,0x26,0xBE,0x03,0x00,0x00,0x45},
{0x28,0xF6,0xD5,0xBD,0x03,0x00,0x00,0xBA},
{0x28,0x4E,0x24,0xBE,0x03,0x00,0x00,0xFF},
{0x28,0xF9,0xFF,0xBD,0x03,0x00,0x00,0xDB},
};  


float pkPa;               // Давление в kPa 

unsigned long currentTime;
unsigned long loopTime;
unsigned long onTime=0;



void setup() {

  lcd.begin(16, 2);   // подключаем дисплей (col,row):
  Serial.begin(9600); // подключаем серийый порт
  pinMode(3, OUTPUT); // устанавливаем 9 ногу как выход тэна 
  pinMode(5, OUTPUT); // устанавливаем 10 ногу как выход тэна
  pinMode(7, OUTPUT); // устанавливаем 10 ногу как выход тэна

}


void loop() {
 

  // Время задерки и круга
  currentTime = millis();        // считываем время, прошедшее с момента запуска программы
  if(currentTime>=onTime){
    digitalWrite(3, LOW);
  }
  if(currentTime >= (loopTime + 500)){   // задержа на 0.5 mc
    loopTime = currentTime;

    /* считываем датчики */
    mpx = (float)analogRead(mpxPin); // читаем данные с MPX
    pkPa = (mpx/(float)1023-0.04)/0.018;//персчитываем в kPa

    //digitalWrite(3, LOW);    // устанавливаем значение на 9 ножке
    //onTime=millis()+100; // высчитали время когда датчик нужно будет выключить. Через 700 милисекунд
  digitalWrite(5, HIGH);
  digitalWrite(7, HIGH);
    /* управление нагревом */
    if (mpx < mpx_a[0]){  // если на датчике меньше чем в первое значение массива
      full_power(); //полный газ
    }

    if (mpx >= mpx_a[0] && mpx <= mpx_a[1]){ //если на датчике больше второго значения массива
      low_power(); //даём ограничение по времени 
    }

    if (mpx > mpx_a[1]){ //если на датчике больше третьего значения массива
      off_power(); // отключаем тен
    }

ds.reset();                // самое интересное - опрос датчиков. Начинается со сброса линии
    ds.write(0xCC,1);          // пропуск ROM - этим мы говорим "Внимание всем!" Следующую команду
    // ... выполняют все устройства (если конечно их тип это позволяет)
    ds.write(0x44,1);          // запускаем конвертацию температуры
    for( n = 1; n < 90; n++) { //ждем 900 мс... (время необходимое датчикам чтобы преобразовать температуру)
      delay(10);            
  }
    for( n = 0; n < 4; n++) {   // Цикл от 0 до 3 - по числу датчиков (4 шт)
      ds.reset();               // начинаем считывать температуры, обмен начинается со сброса
      ds.select(addr[n]);       // выбираем нужный (по порядку) адрес датчика, после этой команды до след. сброса
      // ... отвечать нам будет только выбранный датчик
      ds.write(0xBE);           // команда на считывание ОЗУ датчика
      for ( i = 0; i < 9; i++) {data[i] = ds.read();} // читаем с линии 9 байт
      Tp[n] = float((data[1] << 8) + data[0])/16;     // значение температуры в первых 2-х байтах. Высчитываем температуру
      // ... команда float приводит тип byte (полученный от датчика) к типу с плавающей точкой
      if (Tp[n] == 85) {} else {T[n]=Tp[n];}          // это для того, чтобы при глюке датчика (обычно при переключениях 
        // ... выдача температуры 85 град не приводила к переходу на другой шаг программы. Мы просто возьмем предыдущее значение
        // ... (вот для чего мы его и хранили, ну впрочем можно еще дельту посчитать, вполне можно найти применение)
    }

  

    /* Шлём в серийник */
    Serial.print("  KUB:");
    Serial.print(T[0]);
    Serial.print("  KOLUMN:");
    Serial.print(T[1]);
    Serial.print("  DEF:");
    Serial.print(T[2]);
    Serial.print("  COOLING:");
    Serial.print(T[3]);
    Serial.print("  kPa ");
    Serial.print(pkPa);
    Serial.print("mpx ");
    Serial.println(mpx);
  }
}


void full_power() {
  digitalWrite(3, HIGH);
  onTime=4294967295;
}


void low_power(){
  digitalWrite(3, HIGH);
  onTime=millis()+400; // высчитали время когда датчик нужно будет выключить. Через 400 милисекунд
  digitalWrite(5, HIGH);
    digitalWrite(7, LOW);
}



void off_power() {
  
  digitalWrite(3, LOW);
  digitalWrite(5, LOW);
  digitalWrite(7, LOW);
  onTime=0;
}

